﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using Microsoft.SharePoint.Administration;

namespace RH.Net35WebConfig
{
	class WebConfigModification
	{
		private readonly SPWebApplication _Application;
		private readonly Collection<SPWebConfigModification> _AllModifications;
		private readonly string _Owner;

		#region properties
		public string Owner
		{
			get { return (string.IsNullOrEmpty(_Owner)) ? string.Empty : _Owner; }
		}

		public IList<SPWebConfigModification> GetAllModifications
		{
			get
			{
				return _Application.WebConfigModifications;
			}
		}
		#endregion

		/// <summary>
		/// 
		/// </summary>
		/// <param name="webApplication"></param>
		/// <param name="owner">your assemblyname or something</param>
		internal WebConfigModification(SPWebApplication webApplication, string owner)
		{
			_Application = webApplication;
			_AllModifications = _Application.WebConfigModifications;
			_Owner = owner;
		}

		internal WebConfigModification(SPWebApplication webApplication)
		{
			_Application = webApplication;
			_AllModifications = _Application.WebConfigModifications;
		}

		#region add
		/// <summary>
		/// add the modification, if it has not been added previously
		/// </summary>
		/// <param name="name"></param>
		/// <param name="xpath"></param>
		/// <param name="scriptResource"></param>
		internal void Add(string name, string xpath, string scriptResource)
		{
			var modification = new SPWebConfigModification(name, xpath)
			{
				Owner = Owner,
				Sequence = 0,
				Value = scriptResource,
				Type = SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode
			};

			if (_AllModifications.Contains(modification) == false)
			{
				_AllModifications.Add(modification);
			}
		}

		/// <summary>
		/// add the modification, if it has not been added previously
		/// </summary>
		/// <param name="modification"></param>
		internal void Add(SPWebConfigModification modification)
		{
			if (_AllModifications.Contains(modification) == false)
			{
				_AllModifications.Add(modification);
			}
		}

		/// <summary>
		/// add the modifications, if they have not been added previously
		/// </summary>
		/// <param name="modifications"></param>
		internal void Add(IList<SPWebConfigModification> modifications)
		{
			foreach (var modification in modifications)
			{
				Add(modification);
			}
		}

		/// <summary>
		/// add the section, if it has not been added previously
		/// </summary>
		/// <param name="name"></param>
		/// <param name="xpath"></param>
		internal void AddSection(string name, string xpath)
		{
			var modification = new SPWebConfigModification(name, xpath)
			{
				Owner = Owner,
				Sequence = 0,
				Type = SPWebConfigModification.SPWebConfigModificationType.EnsureSection
			};

			if (_AllModifications.Contains(modification) == false)
			{
				_AllModifications.Add(modification);
			}
		}
		#endregion

		#region remove
		/// <summary>
		/// if the modification exists, it will be removed
		/// </summary>
		/// <param name="name"></param>
		/// <param name="xpath"></param>
		/// <param name="scriptResource"></param>
		internal void Remove(string name, string xpath, string scriptResource)
		{
			var modification = new SPWebConfigModification(name, xpath)
			{
				Owner = Owner,
				Sequence = 0,
				Value = scriptResource
			};

			if (_AllModifications.Contains(modification))
			{
				_AllModifications.Remove(modification);
			}
		}

		/// <summary>
		/// if the modification exists, it will be removed
		/// </summary>
		/// <param name="modification"></param>
		internal void Remove(SPWebConfigModification modification)
		{
			if (_AllModifications.Contains(modification))
			{
				_AllModifications.Remove(modification);
			}
		}

		/// <summary>
		/// if the modifications exists, they will be removed
		/// </summary>
		/// <param name="modifications"></param>
		internal void Remove(IList<SPWebConfigModification> modifications)
		{
			foreach (var modification in modifications)
			{
				Remove(modification);
			}
		}
		#endregion

		#region misc
		/// <summary>
		/// Owner is taken from object.Owner
		/// </summary>
		/// <param name="name"></param>
		/// <param name="xpath"></param>
		/// <param name="scriptResource">will be used as Value</param>
		/// <returns></returns>
		internal SPWebConfigModification Create(string name, string xpath, string scriptResource)
		{
			var modification = new SPWebConfigModification(name, xpath)
			{
				Owner = Owner,
				Sequence = 0,
				Value = scriptResource,
				Type = SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode
			};
			return modification;
		}

		/// <summary>
		/// store modifications in the farm, and let it update the web.config
		/// </summary>
		internal void Save()
		{
			if (_AllModifications.Count == 0) return;

			Logging.WriteULSVerbose("Applying web.config modifications...");
			_Application.Farm.Services.GetValue<SPWebService>().ApplyWebConfigModifications();
			_Application.Update();
			//do not .Clear() the _AllModifications, because you will use your customizations!
		}
		#endregion
	}
}
